/* c't Mikrocontroller-im-LAN Demo Source
 * Datei: uart.c 
 * Info: Routinen zur seriellen Kommunikation
 * Autor: Benjamin Benz (bbe@heise.de)
 * Datum: 24.06.04
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include "uart.h"
#include "led.h"

#define UART_RX_BUFFER_SIZE 128

#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
	#error RX buffer size is not a power of 2
#endif


static unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
static volatile unsigned char UART_RxHead;
static volatile unsigned char UART_RxTail;

char uart_dest_akt =DEST_XPORT; // Hier sichern wir den aktuellen Zustand
char uart_dest_old =DEST_XPORT; // Hier sichern wir den letzten Zustand

void uart_init(void){
	UBRRH= 0x0;  // UART auf 9600 baud
	UBRRL= 0x5F; //  UBRR= (fquarz/ (16* BAUD) ) -1
	
	UCSRC=0x86; // 8 bit 1 Stop No Parity

	UCSRB= ((1<<RXEN) | (1<<TXEN)|(1<<RXCIE)); //Transmit&Receive Enable + RX Int
	

	/* Flush receive buffer */

	UART_RxTail = 0;
	UART_RxHead = 0;
}

/* Interrupt Handler for UART RECV */
SIGNAL (SIG_UART_RECV){
	unsigned char data;
	unsigned char tmphead;
	
	data = UDR;                 /* Read the received data */

	/* Calculate buffer index */
	tmphead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
	UART_RxHead = tmphead;      /* Store new index */

	if ( tmphead == UART_RxTail )
	{
		/* ERROR! Receive buffer overflow */
	}
	
	UART_RxBuf[tmphead] = data; /* Store received data in buffer */
}

char uart_data_available(void){
	if (UART_RxHead == UART_RxTail) 
		return 0;
	else return 1;
}

char uart_read(void){	// Achtung ist noch blockierend!!!!
	unsigned char tmptail;
	while ( UART_RxHead == UART_RxTail ){}
	
	tmptail = ( UART_RxTail + 1 ) & UART_RX_BUFFER_MASK;/* Calculate buffer index */
	
	UART_RxTail = tmptail;                /* Store new index */
	
	return UART_RxBuf[tmptail];           /* Return data */
}

void uart_send(char data){ // Achtung ist noch blockierend!!!!
	LED_on(LED_ROT);

	while (((UCSRA >> UDRE) & 1) ==0){
		
	}	// warten bis UART sendebereit

	LED_off(LED_ROT);
	UDR= data;
}

void uart_dest(char dest){
	if (dest == DEST_OLD)	// Zurck zum letzten Zustand
		dest=uart_dest_old;
	
	switch (dest) {
		case DEST_XPORT: 
			PORTB &= 0xFC;				// PB0=0; PB1=0  ==> Z=Z0 Y=Y0 
			PORTB |= 0x04; 				// PB2=1 ==> (X=X1)	
			break;
		case DEST_COM: 
			PORTB |=  0x03;				// PB0=1; PB1=1  ==> Z=Z1 Y=Y1 
			PORTB &=  0xFB;				// PB2=0 ==> (X=X0) (eigentlich don't care)
			break;
		case DEST_PASSTHROUGH:	
			//PORTB |=  0x03;			//  
			PORTB 	&=  0xFC;			// PB0=1; PB1=1  ==> Z=Z0 Y=Y0 
			break;
	}
	uart_dest_old=uart_dest_akt;
	uart_dest_akt=dest;	// Zustand sichern
}
